from typing import Any, List

import pandas as pd
import streamlit as st
from api.dummy_api import ADummyApi  # type: ignore
from components.meta_data_display import MetaDataDisplay  # type: ignore

from .table_results import TableDisplay
from .tool_tab import ToolTab

from .speech_display_mixin import ExpandedSpeechDisplay


class KWICDisplay(ExpandedSpeechDisplay, ToolTab):
    def __init__(
        self, another_api: ADummyApi, shared_meta: MetaDataDisplay, tab_key: str
    ) -> None:
        super().__init__(another_api, shared_meta, tab_key)

        self.CURRENT_PAGE = f"current_page_{self.TAB_KEY}"
        self.SEARCH_PERFORMED = f"search_performed__{self.TAB_KEY}"
        self.EXPANDED_SPEECH =f"expanded_speech__{self.TAB_KEY}"
        self.ROWS_PER_PAGE = f"rows_per_page__{self.TAB_KEY}"
        self.DATA_KEY = f"data_{self.TAB_KEY}"
        self.SORT_KEY = f"sort_key_{self.TAB_KEY}"
        self.ASCENDING_KEY = f"ascending_{self.TAB_KEY}"
        st.caption(
            "Med verktyget **Key Words in Context** kan du söka på ord och fraser, t.ex. `information` eller `information om`. För att få fler träffar kan `.*` användas, t.ex. `.*information.*`. Under Filtrera sökresultatet kan du avgränsa sökningen till vissa partier, talare eller år. "
        )

        if self.has_and_is(self.EXPANDED_SPEECH):
            self.display_expanded_speech(
                self.get_reset_dict(), self.api, self.TAB_KEY, search_terms=None
            )
        else:
            self.top_container = st.container()
            self.result_desc_container = st.container()
            self.n_hits_container = st.container()
            self.result_container = st.container()

            self.st_dict_when_button_clicked = {
                self.SEARCH_PERFORMED: True,
                self.CURRENT_PAGE: 0,
                self.EXPANDED_SPEECH: False,
            }

            self.define_displays()

            with self.top_container:
                st.text_input("Skriv sökterm:", key=f"search_box_{self.TAB_KEY}")
                self.add_window_size()
                self.add_search_button("Sök")
                self.draw_line()

            self.init_session_state(self.get_initial_values())
            if st.session_state[self.SEARCH_PERFORMED]:
                self.show_display()

    def get_initial_values(self) -> dict:
        return {
            self.SEARCH_PERFORMED: False,
            self.CURRENT_PAGE: 0,
            self.ROWS_PER_PAGE: 5,
        }

    def handle_button_click(self) -> None:
        if not self.handle_search_click(self.st_dict_when_button_clicked):
            st.session_state[self.SEARCH_PERFORMED] = False

    def add_window_size(self) -> None:
        cols_before, cols_after, _ = st.columns([2, 2, 2])
        with cols_before:
            self.add_window_select(
                "Antal ord före sökordet", f"n_words_before_{self.TAB_KEY}"
            )
        with cols_after:
            self.add_window_select(
                "Antal ord efter sökordet", f"n_words_after_{self.TAB_KEY}"
            )

    def add_window_select(self, message: str, key: str):
        st.number_input(
            message,
            key=key,
            min_value=0,
            max_value=5,
            value=2,
        )

    def define_displays(self) -> None:
        self.table_display = TableDisplay(
            current_container_key=self.TAB_KEY,
            current_page_name=self.CURRENT_PAGE,
            party_abbrev_to_color=self.api.party_abbrev_to_color,
            expanded_speech_key=self.EXPANDED_SPEECH,
            rows_per_table_key=self.ROWS_PER_PAGE,
            table_type="kwic",
            data_key=self.DATA_KEY,
        )

    def get_reset_dict(self) -> dict:
        reset = {
            self.SEARCH_PERFORMED: True,
            self.CURRENT_PAGE: st.session_state[self.CURRENT_PAGE],
            self.EXPANDED_SPEECH: False,
        }
        if f"search_box_{self.TAB_KEY}" in st.session_state:
            reset[f"search_box_{self.TAB_KEY}"] = st.session_state[
                f"search_box_{self.TAB_KEY}"
            ]
        else:
            reset[f"search_box_{self.TAB_KEY}"] = self.get_search_box()

        return reset

    def show_display(self) -> None:
        hit = self.get_search_box()
        if hit:
            hits = [h.strip() for h in hit.split(" ")]
            self.show_hit(hits)
            with self.n_hits_container:
                self.add_hits_per_page(self.ROWS_PER_PAGE)
        else:
            self.display_settings_info_no_hits()

    def show_hit(self, hits: List[str]) -> None:
        data = self.get_data(
            hits,
            self.search_display.get_slider(),
            selections=self.search_display.get_selections(),
            words_before=st.session_state[f"n_words_before_{self.TAB_KEY}"],
            words_after=st.session_state[f"n_words_after_{self.TAB_KEY}"],
        )

        if data.empty:
            self.display_settings_info_no_hits()
        else:
            with self.n_hits_container:
                self.add_download_button(data, "kwic.csv")
            with self.result_desc_container:
                self.display_settings_info()
            with self.result_container:
                (
                    left_col,
                    hit_col,
                    right_col,
                    party_col,
                    year_col,
                    link_col,
                    # gender_col,
                    proto_col,
                    # expander_col,
                ) = self.table_display.get_kwick_columns()

                with left_col:
                    st.button(
                        "Vänster",
                        key="sb_left_kwic",
                        on_click=self.set_sorting,
                        args=("Kontext Vänster",),
                    )
                with hit_col:
                    st.button(
                        "Träff",
                        key="sb_hit_kwic",
                        on_click=self.set_sorting,
                        args=("Sökord",),
                    )
                with right_col:
                    st.button(
                        "Höger",
                        key="sb_right_kwic",
                        on_click=self.set_sorting,
                        args=("Kontext Höger",),
                    )
                with party_col:
                    st.button(
                        "Parti",
                        key="sb_party_kwic",
                        on_click=self.set_sorting,
                        args=("Parti",),
                    )
                with year_col:
                    st.button(
                        "År",
                        key="sb_year_kwic",
                        on_click=self.set_sorting,
                        args=("År",)
                    )
                with link_col:

                    
                    st.button(
                        "Talare",
                        key="sb_speaker_kwic",
                        on_click=self.set_sorting,
                        args=("Talare",)
                    )
              
                # with gender_col:
                #    st.markdown("**Kön**")
                #    st.button('↕', key='button könssortering_kwic', on_click=self.set_sorting('Kön',))

                if self.SORT_KEY in st.session_state:
    
                    data.sort_values(
                        st.session_state[self.SORT_KEY],
                        ascending=st.session_state[self.ASCENDING_KEY],
                        inplace=True,
                    )

                st.session_state[self.DATA_KEY] = data
                self.table_display.write_table()

    @st.cache_data
    def get_data(
        _self,
        hits: List[str],
        slider: Any,
        selections: dict,
        words_before: int,
        words_after: int,
    ) -> pd.DataFrame:
        data = _self.api.get_kwic_results_for_search_hits(
            hits,
            from_year=slider[0],
            to_year=slider[1],
            selections=selections,
            words_before=words_before,
            words_after=words_after,
        )

        return data



from typing import Any, Tuple, List

import pandas as pd
import plotly.graph_objects as go  # type: ignore
import streamlit as st
from api.dummy_api import ADummyApi  # type: ignore
from components.meta_data_display import MetaDataDisplay  # type: ignore

from .table_results import TableDisplay
from .tool_tab import ToolTab

from .speech_display_mixin import ExpandedSpeechDisplay


class WordTrendsDisplay(ExpandedSpeechDisplay, ToolTab):
    def __init__(
        self, another_api: ADummyApi, shared_meta: MetaDataDisplay, tab_key: str
    ) -> None:
        super().__init__(another_api, shared_meta, tab_key)
        self.SEARCH_PERFORMED = "search_performed_wt"
        self.CURRENT_PAGE_TAB = "current_page_tab"
        self.CURRENT_PAGE_SOURCE = "current_page_source"
        self.EXPANDED_SPEECH = "expanded_speech_wt"
        self.ROWS_PER_PAGE_TABLE = "rows_per_page_wt_table"
        self.ROWS_PER_PAGE_SOURCE = "rows_per_page_wt_source"
        self.NORMAL_TABLE_WT = "normalize_table_wt"
        self.NORMAL_DIAGRAM_WT = "normalize_diagram_wt"
        self.DISPLAY_SELECT = "wt_display_select"
        self.words_per_year = self.api.get_words_per_year()
        self.SORT_KEY = "sort_key_wt_source"
        self.ASCENDING_KEY = "ascending_wt_source"
        self.DATA_KEY_TABLE = "data_wt_table"
        self.DATA_KEY_SOURCE = "data_wt_source"

        if self.has_and_is(self.EXPANDED_SPEECH):
            self.display_expanded_speech(
                self.get_reset_dict(),
                self.api,
                self.TAB_KEY,
                self.get_search_terms(),
            )
        else:
            st.caption(
                "Sök på ett begrepp för att se hur det använts över tid. För att söka på flera termer, skriv dem med kommatecken, t.ex. debatt,information. Sök med * för att få fler varianter, t.ex. debatt*. Välj vilka talare ska ingå till vänster."
            )

            self.top_container = st.container()
            self.search_container = st.container()
            self.top_result_container = st.container()
            self.result_container = st.container()

            self.st_dict_when_button_clicked = {
                self.SEARCH_PERFORMED: True,
                self.CURRENT_PAGE_TAB: 0,
                self.CURRENT_PAGE_SOURCE: 0,
            }

            self.init_session_state(self.get_initial_values())
            self.define_displays()

            with self.search_container:
                st.text_input("Skriv sökterm:", key=f"search_box_{self.TAB_KEY}")

                self.add_search_button("Sök")

            if st.session_state[self.SEARCH_PERFORMED]:
                self.show_display()

    def get_initial_values(self):
        session_state_initial_values = {
            self.SEARCH_PERFORMED: False,
            self.CURRENT_PAGE_TAB: 0,
            self.CURRENT_PAGE_SOURCE: 0,
            self.ROWS_PER_PAGE_SOURCE: 5,
            self.ROWS_PER_PAGE_TABLE: 5,
            self.NORMAL_TABLE_WT: "Frekvens",
            self.NORMAL_DIAGRAM_WT: "Frekvens",
        }

        return session_state_initial_values

    def get_reset_dict(self) -> dict:
        return {
            self.SEARCH_PERFORMED: True,
            self.CURRENT_PAGE_SOURCE: st.session_state[self.CURRENT_PAGE_SOURCE],
            f"search_box_{self.TAB_KEY}": self.get_current_search_as_str(),
            self.DISPLAY_SELECT: "Anföranden",
            self.EXPANDED_SPEECH: False,
        }

    def get_current_search_as_str(self) -> str:
        if f"search_box_{self.TAB_KEY}" in st.session_state:
            return st.session_state[f"search_box_{self.TAB_KEY}"]
        return ""

    def handle_button_click(self) -> None:
        if not self.handle_search_click(self.st_dict_when_button_clicked):
            st.session_state[self.SEARCH_PERFORMED] = False

    def define_displays(self) -> None:
        self.table_display_table = TableDisplay(
            current_container_key="WT_TABLE",
            current_page_name=self.CURRENT_PAGE_TAB,
            party_abbrev_to_color=self.api.party_abbrev_to_color,
            expanded_speech_key="",
            rows_per_table_key=self.ROWS_PER_PAGE_TABLE,
            table_type="table",
            data_key=self.DATA_KEY_TABLE,
        )
        self.table_display = TableDisplay(
            current_container_key="WT_SOURCE",
            current_page_name=self.CURRENT_PAGE_SOURCE,
            party_abbrev_to_color=self.api.party_abbrev_to_color,
            expanded_speech_key=self.EXPANDED_SPEECH,
            rows_per_table_key=self.ROWS_PER_PAGE_SOURCE,
            table_type="source",
            data_key=self.DATA_KEY_SOURCE,
        )

    @st.cache_data
    def get_data(
        _self,
        search_words: List[str],
        start_year: int,
        end_year: int,
        selections: dict,
    ) -> Tuple[pd.DataFrame, pd.DataFrame]:

        return _self.api.get_word_trend_results(
            search_words,
            filter_opts=selections,
            start_year=start_year,
            end_year=end_year,
        )
    


    def normalize_word_per_year(self, data: pd.DataFrame) -> pd.DataFrame:
        data = data.merge(self.words_per_year, left_index=True, right_index=True)
        data = data.iloc[:, :].div(data.n_raw_tokens, axis=0)
        data.drop(columns=["n_raw_tokens"], inplace=True)

        return data

    def show_display(self) -> None:
        slider = st.session_state["years_slider"]

        selections = self.search_display.get_selections()
        hits = self.get_search_terms()

        if len(hits) > 0:
            with self.search_container:
                hit_selector = st.multiselect(
                    label="Välj sökord att inkludera", options=hits, default=hits
                )

            data, kwic_like_data = self.get_data(
                hit_selector,
                slider[0],
                slider[1],
                selections,
            )

            with self.top_result_container:
                self.draw_line()
                if kwic_like_data.empty:
                    self.display_settings_info_no_hits()
                else:
                    self.display_settings_info(hits=f": {', '.join(hits)}")

                    col_display_select, _, col_page_select = st.columns([2, 1, 1])
                    with col_display_select:
                        self.add_radio_buttons()
                    self.add_download_button(data, "word_trends.csv")
                    self.draw_line()
                    self.display_results(data, kwic_like_data, col_page_select)
        else:
            st.info(
                f"Inga träffar för **{self.get_search_box()}**, försök med andra sökord."
            )

    def add_radio_buttons(self):
        st.radio(
            "Visa resultat som:",
            ["Diagram", "Tabell", "Anföranden"],
            index=0,
            key=self.DISPLAY_SELECT,
            help=None,
            horizontal=True,
        )

    def get_search_terms(self):
        search_terms = self.get_search_box().split(",")
        search_terms = [term.strip().lower() for term in search_terms]

        hits = self.parse_search_string(search_terms)
        return hits

    def parse_search_string(self, search_terms):
        hits = []
        if len(search_terms) > 0:
            for term in search_terms:
                current_hits = self.api.get_word_hits(term, n_hits=10)
                for hit in current_hits:
                    hits.append(hit)
                hits.extend(current_hits)
        hits = list(set(hits))
        return hits

    def normalize(self, data, normalization_key):
        self.radio_normalize(normalization_key)
        normalize = st.session_state[normalization_key] == "Normaliserad frekvens"
        if normalize:
            data = self.normalize_word_per_year(data)
        return data

    def display_results(
        self, data: pd.DataFrame, kwic_like_data: pd.DataFrame, col_page_select: Any
    ) -> None:
        with self.result_container:
            if st.session_state[self.DISPLAY_SELECT] == "Diagram":
                data = self.normalize(data, self.NORMAL_DIAGRAM_WT)
                self.draw_line_figure(data)

            elif st.session_state[self.DISPLAY_SELECT] == "Tabell":
                data = self.normalize(data, self.NORMAL_TABLE_WT)
                st.session_state[self.DATA_KEY_TABLE] = data
                with col_page_select:
                    self.add_hits_per_page(self.ROWS_PER_PAGE_TABLE)
                self.table_display_table.write_table()

            else:
                with col_page_select:
                    self.add_hits_per_page(self.ROWS_PER_PAGE_SOURCE)

                self.add_anforande_display(kwic_like_data)

    def add_anforande_display(self, kwic_like_data):
        (
            speaker_col,
            gender_col,
            year_col,
            party_col,
            link_col,
            expander_col,
        ) = self.table_display.get_columns()

        with speaker_col:
            st.button(
                "Talare↕", key="sb_t_wt", on_click=self.set_sorting, args=("Talare",)
            )
        with gender_col:
            st.button("Kön↕", key="sb_k_wt", on_click=self.set_sorting, args=("Kön",))
        with year_col:
            st.button("År↕", key="sb_y_wt", on_click=self.set_sorting, args=("År",))
        with party_col:
            st.button(
                "Parti↕", key="sb_p_wt", on_click=self.set_sorting, args=("Parti",)
            )
        with link_col:
            st.button(
                "Källa↕", key="sb_l_wt", on_click=self.set_sorting, args=("Protokoll",)
            )

        with expander_col:
            st.write("Tal")

        if self.SORT_KEY in st.session_state:
            kwic_like_data.sort_values(
                st.session_state[self.SORT_KEY],
                ascending=st.session_state[self.ASCENDING_KEY],
                inplace=True,
            )

        st.session_state[self.DATA_KEY_SOURCE] = kwic_like_data
        self.table_display.write_table()

    def radio_normalize(self, key):
        st.radio(
            "Normalisera resultatet?",
            ("Frekvens", "Normaliserad frekvens"),
            key=key,
            help="Frekvens: antal förekomster av söktermen per år. Normaliserad frekvens: antal förekomster av söktermen delat med totalt antal ord i tal under samma år.",
            horizontal=True,
        )

    def draw_line_figure(self, data: pd.DataFrame) -> None:
        markers = ["circle", "hourglass", "x", "cross", "square", 5]
        lines = ["solid", "dash", "dot", "dashdot", "solid"]
        fig = go.Figure()
        for i, col in enumerate(data.columns):
            fig.add_trace(
                go.Scatter(
                    x=data.index,
                    y=data[col],
                    mode="lines+markers",
                    name=col,
                    marker=dict(symbol=markers[i % len(markers)], size=8),
                    line=dict(dash=lines[i % len(lines)]),
                    showlegend=True,
                )
            )
        fig.update_yaxes(exponentformat="none")
        fig.update_layout(xaxis_title="År", yaxis_title="Frekvens")
        fig.update_layout(separators=",   ")
        st.plotly_chart(fig, use_container_width=True, theme="streamlit")



from .tool_tab import ToolTab
from .speech_display_mixin import ExpandedSpeechDisplay
from typing import Any

import pandas as pd
import streamlit as st
from api.dummy_api import ADummyApi  # type: ignore
from components.meta_data_display import MetaDataDisplay  # type: ignore

from .table_results import TableDisplay



class FullSpeechDisplay(ExpandedSpeechDisplay, ToolTab):
    def __init__(
        self, another_api: ADummyApi, shared_meta: MetaDataDisplay, tab_key: str
    ) -> None:
        super().__init__(another_api, shared_meta, tab_key)

        self.CURRENT_PAGE = f"current_page_{self.TAB_KEY}"
        self.SEARCH_PERFORMED = f"search_performed_{self.TAB_KEY}"
        self.EXPANDED_SPEECH = f"expanded_speech_{self.TAB_KEY}"
        self.ROWS_PER_PAGE = f"rows_per_page_{self.TAB_KEY}"
        self.DATA_KEY = f"data_{self.TAB_KEY}"
        self.SORT_KEY = f"sort_key_{self.TAB_KEY}"
        self.ASCENDING_KEY = f"ascending_{self.TAB_KEY}"

        if self.has_and_is(self.EXPANDED_SPEECH):
            reset_dict = {self.EXPANDED_SPEECH: False}
            self.display_expanded_speech(reset_dict, self.api, self.TAB_KEY)
        else:
            session_state_initial_values = {
                self.CURRENT_PAGE: 0,
                self.SEARCH_PERFORMED: False,
                self.EXPANDED_SPEECH: False,
                self.ROWS_PER_PAGE: 5,
            }
            self.st_dict_when_button_clicked = {
                self.CURRENT_PAGE: 0,
                self.SEARCH_PERFORMED: True,
                self.EXPANDED_SPEECH: False,
            }

            st.caption(
                "Sök på hela anföranden. Välj tidsintervall, partier, kön och talare till vänster."
            )
            self.top_container = st.container()
            self.bottom_container = st.container()

            self.table_display = TableDisplay(
                current_container_key=self.TAB_KEY,
                current_page_name=self.CURRENT_PAGE,
                party_abbrev_to_color=self.api.party_abbrev_to_color,
                expanded_speech_key=self.EXPANDED_SPEECH,
                rows_per_table_key=self.ROWS_PER_PAGE,
                table_type="source",
                data_key=self.DATA_KEY,
            )

            with self.top_container:
                self.add_search_button("Visa anföranden")
                self.draw_line()

            self.init_session_state(session_state_initial_values)

            if st.session_state[self.SEARCH_PERFORMED]:
                self.show_display()

    def handle_button_click(self) -> None:
        for k, v in self.st_dict_when_button_clicked.items():
            st.session_state[k] = v

    @st.cache_data
    def get_anforanden(
        _self, _another_api: Any, from_year: int, to_year: int, selections: dict
    ) -> pd.DataFrame:
        data = _another_api.get_anforanden(from_year, to_year, selections)
        return data

    def show_display(self) -> None:
        start_year, end_year = self.search_display.get_slider()

        anforanden = self.get_anforanden(
            self.api,
            start_year,
            end_year,
            selections=self.search_display.get_selections(),
        )
        if len(anforanden) == 0:
            self.display_settings_info_no_hits(with_search_hits=False)
        else:
            self.display_results(anforanden)

    def display_results(self, anforanden: pd.DataFrame) -> None:
        with self.bottom_container:
            self.display_settings_info(with_search_hits=False)
            _, col_right = st.columns([4, 2])

            with col_right:
                self.add_hits_per_page(self.ROWS_PER_PAGE)
                self.add_download_button(anforanden, "anforanden.csv")
            self.draw_line()

            (
                speaker_col,
                gender_col,
                year_col,
                party_col,
                link_col,
                expander_col,
            ) = self.table_display.get_columns()

            with speaker_col:
                st.button(
                    "Talare↕",
                    key="sort_button",
                    on_click=self.set_sorting,
                    args=("Talare",),
                )
            with gender_col:
                st.button(
                    "Kön↕",
                    key="button könssortering",
                    on_click=self.set_sorting,
                    args=("Kön",),
                )
            with year_col:
                st.button(
                    "År↕",
                    key="button årsortering",
                    on_click=self.set_sorting,
                    args=("År",),
                )
            with party_col:
                st.button(
                    "Parti↕",
                    key="button partisortering",
                    on_click=self.set_sorting,
                    args=("Parti",),
                )
            with link_col:
                st.button(
                    "Källa↕",
                    key="button protokollsortering",
                    on_click=self.set_sorting,
                    args=("Protokoll",),
                )

            if self.SORT_KEY in st.session_state:
                anforanden.sort_values(
                    st.session_state[self.SORT_KEY],
                    ascending=st.session_state[self.ASCENDING_KEY],
                    inplace=True,
                )

            st.session_state[self.DATA_KEY] = anforanden
            self.table_display.write_table()



import streamlit as st
from swedeb_demo.api.dummy_api import ADummyApi  # type: ignore
from typing import List


class ExpandedSpeechDisplay:
    def display_expanded_speech(
        self,
        reset_dict: dict,
        api: ADummyApi,
        tab_key: str,
        search_terms: List[str] = None,
    ) -> None:
        
        selected_protocol = st.session_state["selected_protocol"]
        
        chamber = 'Andra kammaren' if 'ak' in selected_protocol else 'Första kammaren'
        
        simplified_protocol = selected_protocol.split("-")[1] + ":" + selected_protocol.split("-")[5].split('_')[0]
        dummy_pdf = "https://www.riksdagen.se/sv/sok/?avd=dokument&doktyp=prot"
        link = f"[{simplified_protocol}]({dummy_pdf})"
        col_1, col_3 = st.columns([3, 1])
        
            #**Anförande**: {selected_protocol}
        with col_1:
            
            info_text = f"""
            **Talare**: {st.session_state['selected_speaker']}    
            **År**: {st.session_state['selected_year']}  

            **Hela protokollet** {link}, {chamber}
            
            **Talarnotering**: {api.get_speaker_note(selected_protocol)}(
            
            
            """
            st.info(info_text)

        with col_3:
            st.button(
                "Stäng och återgå",
                key=f"close_button_{tab_key}",
                on_click=self.reset_speech_state,
                args=(reset_dict,),
            )
        st.write('**Hela anförandet:**')
        text = api.get_speech_text(st.session_state["selected_protocol"])
        text = text.replace("\n", "<br><br>")
        if search_terms is not None:
            for search_term in search_terms:
                text = text.replace(
                    search_term,
                    f'<span style="background-color: #FFFF00">{search_term}</span>',
                )
        st.markdown(
            f'<p style="border-width:2px; border-style:solid; border-color:#000000; padding: 1em;">{text}</p>',
            unsafe_allow_html=True,
        )



    def reset_speech_state(self, reset_dict: dict) -> None:
        st.session_state["selected_protocol"] = None
        for k, v in reset_dict.items():
            st.session_state[k] = v



"""_summary_: This is the main page of the SweDeb app with a global meta data filter and a tabbed interface for the different tools.
"""
import streamlit as st
from api.dummy_api import ADummyApi  # type: ignore
from components.kwic_tab import KWICDisplay  # type: ignore
from components.meta_data_display import MetaDataDisplay  # type: ignore
from components.whole_speeches_tab import FullSpeechDisplay  # type: ignore
from components.word_trends_tab import WordTrendsDisplay  # type: ignore
from typing import Any
import click

st.set_page_config(
    page_title="SweDeb DEMO b",
    layout="wide",
)


def add_banner() -> None:
    """_summary_: Adds a banner to the top of the page containing the title and a dropdown menu for selecting a corpus"""
    banner = st.container()
    with banner:
        other_col, corpus_col = st.columns([2, 1])
        with corpus_col:
            st.selectbox(
                "Välj korpus",
                (["Sample data"]),
                key="corpus_box",
                help="Välj vilket korpus du vill arbeta med",
                index=0,
            )
        with other_col:
            st.title("Svenska riksdagsdebatter")


def set_swedish_for_selections() -> None:
    SELECT_TEXT = "Välj ett eller flera alternativ"
    multi_css = f"""
    <style>
    .stMultiSelect div div div div div:nth-of-type(2) {{visibility: hidden;}}
    .stMultiSelect div div div div div:nth-of-type(2)::before {{visibility: visible; content:"{SELECT_TEXT}";}}
    </style>
    """
    st.markdown(multi_css, unsafe_allow_html=True)


def add_meta_sidebar(api: ADummyApi) -> Any:
    sidebar_container = st.sidebar.container()
    with sidebar_container:
        st.header("Filtrera sökresultat")
        side_expander = st.expander("Visa filtreringsalternativ:")
        with side_expander:
            meta_search = MetaDataDisplay(st_dict={}, api=api)
    return meta_search


def add_tabs(meta_search: Any, api: ADummyApi, debug: bool) -> None:
    debug_tab = " "
    if debug:
        debug_tab = "Debug"

    (
        tab_WT,
        tab_KWIC,
        tab_whole_speeches,
        tab_NG,
        tab_topics,
        tab_about,
        tab_debug,
    ) = st.tabs(
        [
            "Ordtrender",
            "KWIC",
            "Anföranden",
            "N-gram",
            "Temamodeller",
            "Om SweDeb",
            debug_tab,
        ]
    )

    with tab_WT:
        WordTrendsDisplay(api, shared_meta=meta_search, tab_key="WT")

    with tab_KWIC:
        KWICDisplay(api, shared_meta=meta_search, tab_key="KWIC")

    with tab_NG:
        st.caption("Här kommer information om verktyget **N-gram**")

    with tab_whole_speeches:
        FullSpeechDisplay(api, shared_meta=meta_search, tab_key="SPEECH")

    with tab_topics:
        st.caption("Här kommer information om verktyget **Temamodeller**")

    with tab_about:
        st.write("Här kommer information om SweDebprojektet och verktygen")
        faq_expander = st.expander("FAQ")
        with faq_expander:
            st.write("Vad är detta?")
            st.caption(
                "En prototyp utvecklad för att undersöka möjligheterna med att tillgängliggöra riksdagsdebatter"
            )
            st.write("Hur använder man den här prototypen?")
            st.caption(
                "I vänsterspalten kan du välja vilken data du vill undersöka. I de olika flikarna kan du välja vilket verktyg du vill använda för att undersöka materialet."
            )
    if debug:
        with tab_debug:
            st.caption("Session state:")
            st.write(st.session_state)


@click.command()
@click.option('--env_file', default='.env_sample_docker', help='Path to .env file with environment variables')
@click.option('--debug', default=True, help='Show session state info in debug tab')
def render_main_page(env_file:str, debug:bool)->None:
    dummy_api = ADummyApi(env_file)
    add_banner()
    set_swedish_for_selections()
    meta_search = add_meta_sidebar(dummy_api)
    add_tabs(meta_search, dummy_api, debug)


if __name__ == "__main__":
    render_main_page()  # type: ignore



from __future__ import annotations
import streamlit as st


class GenderCheckBoxes:
    def __init__(self) -> None:
        st.checkbox(
            "Filtrera på kön?",
            value=False,
            key="use_gender_filter",
            on_change=self.set_box_values,
        )
        disabled = not st.session_state["use_gender_filter"]

        st.checkbox("Män", disabled=disabled, key="gender_check_men")
        st.checkbox("Kvinnor", disabled=disabled, key="gender_check_women")
        st.checkbox("Kön okänt", disabled=disabled, key="gender_check_unknown")

    def set_box_values(self) -> None:
        st.session_state["gender_check_unknown"] = True
        st.session_state["gender_check_women"] = True
        st.session_state["gender_check_men"] = True

    def get_selection(self) -> dict | None:
        if st.session_state["use_gender_filter"]:
            selected = []
            if st.session_state["gender_check_men"]:
                selected.append(1)
            if st.session_state["gender_check_women"]:
                selected.append(2)
            if st.session_state["gender_check_unknown"]:
                selected.append(0)
            if selected:
                return {"gender_id": selected}
        return None

    def get_selected_genders_string(self) -> str:
        if st.session_state["use_gender_filter"]:
            selected = []

            if st.session_state["gender_check_men"]:
                selected.append("Män")

            if st.session_state["gender_check_women"]:
                selected.append("Kvinnor")

            if st.session_state["gender_check_unknown"]:
                selected.append("Okänt kön")

            gender_str = ", ".join(selected)
            if not selected:
                return f"Välj kön i menyn till vänster för att visa resultat för enskilda grupper  \n"
            return f"**Valda kön**: {gender_str}.\n"
        return "**Valda kön**: Alla"



from typing import Any

import pandas as pd
import streamlit as st
from api.dummy_api import ADummyApi  # type: ignore
from components.meta_data_display import MetaDataDisplay  # type: ignore


class ToolTab:
    def __init__(
        self, another_api: ADummyApi, shared_meta: MetaDataDisplay, tab_key: str
    ):
        self.api = another_api
        self.search_display = shared_meta
        self.TAB_KEY = tab_key
        self.HITS_PER_PAGE = f"{self.TAB_KEY}_hits_per_page"


    def init_session_state(self, session_dict: dict) -> None:
        for k, v in session_dict.items():
            if k not in st.session_state:
                st.session_state[k] = v

    def get_search_box(self) -> str:
        if f"search_box_{self.TAB_KEY}" not in st.session_state:
            return ""
        return st.session_state[f"search_box_{self.TAB_KEY}"]

    def handle_search_click(self, st_dict_when_button_clicked: dict) -> bool:
        if self.get_search_box() != "":
            for k, v in st_dict_when_button_clicked.items():
                st.session_state[k] = v
            return True
        else:
            st.warning("Fyll i en sökterm")
            return False

    def display_settings_info(
        self, with_search_hits: bool = True, hits: str = ""
    ) -> None:
        if with_search_hits:
            st.info(
                f"Resultat för sökningen **_{self.get_search_box()}_** {hits}  \n{self.search_display.get_current_settings()}"
            )
        else:
            st.info(
                f"Resultat för sökningen:  \n{self.search_display.get_current_settings()}"
            )

    def display_settings_info_no_hits(self, with_search_hits: bool = True) -> None:
        if with_search_hits:
            st.info(
                f"Inga resultat för sökningen **_{self.get_search_box()}_**.  \n{self.search_display.get_current_settings()}  \nUtöka filtreringen eller försök med ett annat sökord för att få fler träffar."
            )
        else:
            st.info(
                f"Inga resultat för sökningen:  \n{self.search_display.get_current_settings()}.  \nUtöka filtreringen för att få fler träffar."
            )

    def add_hit_selector(self, hits: list) -> Any:
        hit_selector = st.multiselect(
            label="Välj ord att inkludera",
            options=hits,
            default=hits,
        )

        return hit_selector

    def draw_line(self) -> None:
        st.markdown(
            """<hr style="height:2px;border:none;color:#111111;background-color:#111111;" /> """,
            unsafe_allow_html=True,
        )

    @st.cache_data
    def convert_df(_self, df: pd.DataFrame) -> bytes:
        return df.to_csv(index=True).encode("utf-8")

    def add_download_button(self, data: pd.DataFrame, file_name: str) -> None:
        st.download_button(
            label="Ladda ner som csv",
            data=self.convert_df(data),
            file_name=file_name,
            mime="text/csv",
        )

    def get_sort_direction(self, key) -> None:
        if key not in st.session_state:
            st.session_state[key] = True
        else:
            st.session_state[key] = not st.session_state[key]
        return st.session_state[key]

    def add_hits_per_page(self, key):
        st.selectbox(
            "Antal resultat per sida",
            options=[5, 10, 20, 50],
            key=key,
        )

    def has_and_is(self, key):
        if key not in st.session_state:
            return False
        return st.session_state[key]

    def set_sorting(self, sorting_key):
        st.session_state[self.ASCENDING_KEY] = self.get_sort_direction(
            f"{sorting_key}_{self.TAB_KEY}"
        )
        st.session_state[self.SORT_KEY] = sorting_key
        self.table_display.reset_page()

    def add_search_button(self, message: str) -> None:
        st.button(
            message,
            key=f"search_button_{self.TAB_KEY}",
            on_click=self.handle_button_click,
        )
